/********************************** (C) COPYRIGHT *******************************
* File Name          : usb_prop.c
* Author             : WCH
* Version            : V1.0.0
* Date               : 2019/10/15
* Description        : All processing related to Virtual Com Port Demo
*******************************************************************************/ 
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_desc.h"
#include "usb_pwr.h"
#include "hw_config.h"
#include "ch32f10x_usb_host.h"
uint8_t Request = 0;
uint8_t PacketSize;
u32 ProtocolValue;

/* -------------------------------------------------------------------------- */
/*  Structures initializations */
/* -------------------------------------------------------------------------- */


DEVICE Device_Table =
{
	EP_NUM,
	1
};

DEVICE_PROP Device_Property =
{
	USBD_init,
	USBD_Reset,
	USBD_Status_In,
	USBD_Status_Out,
	USBD_Data_Setup,
	USBD_NoData_Setup,
	USBD_Get_Interface_Setting,
	USBD_GetDeviceDescriptor,
	USBD_GetConfigDescriptor,
	USBD_GetStringDescriptor,
	0,
//	0x40
};

USER_STANDARD_REQUESTS User_Standard_Requests =
{
	USBD_GetConfiguration,
	USBD_SetConfiguration,
	USBD_GetInterface,
	USBD_SetInterface,
	USBD_GetStatus, 
	USBD_ClearFeature,
	USBD_SetEndPointFeature,
	USBD_SetDeviceFeature,
	USBD_SetDeviceAddress
};

ONE_DESCRIPTOR Device_Descriptor =
{
	(uint8_t*)USBD_DeviceDescriptor,
	USBD_SIZE_DEVICE_DESC
};

ONE_DESCRIPTOR Config_Descriptor =
{
	(uint8_t*)USBD_ConfigDescriptor,
	USBD_SIZE_CONFIG_DESC
};

/*******************************************************************/
																	//
ONE_DESCRIPTOR Interface0Report_Descriptor =							//
  {																	//
    (u8 *)Interface0RepDesc,
    ReportDescriptor_Size										//
  };																//
ONE_DESCRIPTOR Interface1Report_Descriptor =                            //
  {                                                                 //
    (u8 *)Interface1RepDesc,
    ReportDescriptor_Size                                       //
  };
ONE_DESCRIPTOR Interface2Report_Descriptor =                            //
  {                                                                 //
    (u8 *)Interface2RepDesc,
    ReportDescriptor_Size                                       //
  };    //
ONE_DESCRIPTOR Interface0Hid_Descriptor =								//
  {																	//
    (u8*)USBD_ConfigDescriptor + Interface0_OFF_HID_DESC,			//
    USBD_SIZ_HID_DESC											//
  };
ONE_DESCRIPTOR Interface1Hid_Descriptor =                               //
  {                                                                 //
    (u8*)USBD_ConfigDescriptor + Interface1_OFF_HID_DESC,          //
    USBD_SIZ_HID_DESC                                           //
  };
ONE_DESCRIPTOR Interface2Hid_Descriptor =                               //
  {                                                                 //
    (u8*)USBD_ConfigDescriptor + Interface2_OFF_HID_DESC,          //
    USBD_SIZ_HID_DESC                                           //
  };
/*******************************************************************/

ONE_DESCRIPTOR String_Descriptor[4] =
{
	{(uint8_t*)USBD_StringLangID, USBD_SIZE_STRING_LANGID},
	{(uint8_t*)USBD_StringVendor, USBD_SIZE_STRING_VENDOR},
	{(uint8_t*)USBD_StringProduct,USBD_SIZE_STRING_PRODUCT},
	{(uint8_t*)USBD_StringSerial, USBD_SIZE_STRING_SERIAL}
};


/*******************************************************************************
* Function Name  : USBD_SetConfiguration.
* Description    : Update the device state to configured.
* Input          : None.
* Return         : None.
*******************************************************************************/
void USBD_SetConfiguration(void)
{
  DEVICE_INFO *pInfo = &Device_Info;

  if (pInfo->Current_Configuration != 0)
  {
		 /* Device configured */
    bDeviceState = CONFIGURED;
  }
}

/*******************************************************************************
* Function Name  : USBD_SetDeviceAddress.
* Description    : Update the device state to addressed.
* Input          : None.
* Return         : None.
*******************************************************************************/
void USBD_SetDeviceAddress (void)
{
  bDeviceState = ADDRESSED;
}

/*******************************************************************************
* Function Name  : USBD_Status_In.
* Description    : USBD Status In Routine.
* Input          : None.
* Return         : None.
*******************************************************************************/
void USBD_Status_In(void)
{
	 if (Request == SET_LINE_CODING)
  {
    Request = 0;
  }
}

/*******************************************************************************
* Function Name  : USBD_Status_Out
* Description    : USBD Status OUT Routine.
* Input          : None.
* Return         : None.
*******************************************************************************/
void USBD_Status_Out(void)
{
    
}

/*******************************************************************************
* Function Name  : USB_init.
* Description    : init routine.
* Input          : None.
* Return         : None.
*******************************************************************************/
void USBD_init(void)
{
  uint8_t	i;
    
  pInformation->Current_Configuration = 0;
	/* Connect the device */
  PowerOn();//ִ꣬豸⵽ܹӦλж
  for (i=0;i<8;i++) _SetENDPOINT(i,_GetENDPOINT(i) & 0x7F7F & EPREG_MASK);//all clear
  _SetISTR((uint16_t)0x00FF);//all clear
	/* Perform basic device initialization operations */
  USB_SIL_Init();
  bDeviceState = UNCONNECTED;
}

/*******************************************************************************
* Function Name  : USBD_Reset
* Description    : Virtual_Com_Port Mouse reset routine
* Input          : None.
* Return         : None.
*******************************************************************************/
void USBD_Reset(void)
{
  pInformation->Current_Configuration = 0;//ǰΪ0
  pInformation->Current_Feature = USBD_ConfigDescriptor[7];
  pInformation->Current_Interface = 0;//ǰӿΪ0
	//Ҫ߹
  SetBTABLE(BTABLE_ADDRESS);//ðַ

	 /* Initialize Endpoint 0 */
  SetEPType(ENDP0, EP_CONTROL);//˵0Ϊƶ˵
  SetEPTxStatus(ENDP0, EP_TX_STALL);//˵״̬ΪЧINʱһSTALL
  SetEPRxAddr(ENDP0, ENDP0_RXADDR);//ö˵0ջַյֽͻַ
  SetEPTxAddr(ENDP0, ENDP0_TXADDR);//ͻַ
  Clear_Status_Out(ENDP0);//EP_KINDSTATUS_OUTλΪãڿģʽֻ0ֽݰӦSTALL
  SetEPRxCount(ENDP0, USBD_DeviceDescriptor[7]);//ջ֧64ֽDevice_Property.MaxPacketSize
//  printf("Device_Property.MaxPacketSize=%d\n",Device_Property.MaxPacketSize);
  SetEPRxValid(ENDP0);
//  _ClearDTOG_RX(ENDP0);
//  _ClearDTOG_TX(ENDP0);

	 /* Initialize Endpoint In 1 */
  SetEPType(ENDP1, EP_INTERRUPT);//ʼΪж϶˵
  SetEPTxAddr(ENDP1, ENDP1_TXADDR);//÷ݵַ
	SetEPTxCount(ENDP1, InEndpointMaxSize);//÷ͳ
  SetEPTxStatus(ENDP1, EP_TX_NAK);//ö˵㴦æ״̬
  SetEPRxStatus(ENDP1, EP_RX_DIS);
  _ClearDTOG_RX(ENDP1);
  _ClearDTOG_TX(ENDP1);
	 /* Initialize Endpoint 2 */
  SetEPType(ENDP2, EP_INTERRUPT);
  SetEPTxAddr(ENDP2, ENDP2_TXADDR);  
  SetEPTxCount(ENDP2, InEndpointMaxSize);
  SetEPRxStatus(ENDP2, EP_RX_DIS);
  SetEPTxStatus(ENDP2, EP_TX_NAK);
  _ClearDTOG_RX(ENDP2);
  _ClearDTOG_TX(ENDP2);

	 /* Initialize Endpoint 3 */
  SetEPType(ENDP3, EP_INTERRUPT);
  SetEPRxAddr(ENDP3, ENDP3_RXADDR);
  SetEPRxCount(ENDP3, USBD_DATA_SIZE);
  SetEPRxStatus(ENDP3, EP_RX_VALID);
  SetEPTxStatus(ENDP3, EP_TX_DIS);
  _ClearDTOG_RX(ENDP3);
  _ClearDTOG_TX(ENDP3);
   
  SetDeviceAddress(0);
  
  bDeviceState = ATTACHED;
}

/*******************************************************************************
* Function Name  : USBD_GetDeviceDescriptor.
* Description    : Gets the device descriptor.
* Input          : Length.
* Return         : The address of the device descriptor.
*******************************************************************************/
uint8_t *USBD_GetDeviceDescriptor(uint16_t Length)
{
  return Standard_GetDescriptorData(Length, &Device_Descriptor);
}

/*******************************************************************************
* Function Name  : USBD_GetConfigDescriptor.
* Description    : get the configuration descriptor.
* Input          : Length.
* Return         : The address of the configuration descriptor.
*******************************************************************************/
uint8_t *USBD_GetConfigDescriptor(uint16_t Length)
{
  return Standard_GetDescriptorData(Length, &Config_Descriptor);
}

/*******************************************************************************
* Function Name  : USBD_GetStringDescriptor
* Description    : Gets the string descriptors according to the needed index
* Input          : Length.
* Return         : The address of the string descriptors.
*******************************************************************************/
uint8_t *USBD_GetStringDescriptor(uint16_t Length)
{
  uint8_t wValue0 = pInformation->USBwValue0;
	
  if (wValue0 > 4)
  {
    return NULL;
  }
  else
  {
    return Standard_GetDescriptorData(Length, &String_Descriptor[wValue0]);
  }
}

/*******************************************************************************
* Function Name  : USBD_Get_Interface_Setting.
* Description    : test the interface and the alternate setting according to the
*                  supported one.
* Input1         : Interface: interface number.
*                  AlternateSetting: Alternate Setting number.
* Return         : The address of the string descriptors.
*******************************************************************************/
RESULT USBD_Get_Interface_Setting(uint8_t Interface, uint8_t AlternateSetting)
{
  if (AlternateSetting > 0)
  {
    return USB_UNSUPPORT;
  }
  else if (Interface > 1)
  {
    return USB_UNSUPPORT;
  }
	
  return USB_SUCCESS;
}

//uint8_t USBD_Get_MaxPacketSize()
//{
//  return USBD_DeviceDescriptor[7];
//}


/*******************************************************************************
* Function Name  : USBD_Data_Setup
* Description    : handle the data class specific requests
* Input          : Request Nb.
* Return         : USB_UNSUPPORT or USB_SUCCESS.
*******************************************************************************/
RESULT USBD_Data_Setup(uint8_t RequestNo)
{
   u8    *(*CopyRoutine)(u16);

  CopyRoutine = NULL;

  if ((RequestNo == GET_DESCRIPTOR)
      && (Type_Recipient == (STANDARD_REQUEST | INTERFACE_RECIPIENT))
      && (pInformation->USBwIndex0 < 2))
  {

    if (pInformation->USBwValue1 == SET_CONTROL_LINE_STATE)
    {
        if (pInformation->USBwIndex0 == 0)
            CopyRoutine = Interface0GetReportDescriptor;
        else if(pInformation->USBwIndex0 == 1)
            CopyRoutine = Interface1GetReportDescriptor;
        else
            CopyRoutine = Interface2GetReportDescriptor;
    }
    else if (pInformation->USBwValue1 == HID_DESCRIPTOR_TYPE)
    {
        if (pInformation->USBwIndex0 == 0)
            CopyRoutine = Interface0GetHIDDescriptor;
        else if(pInformation->USBwIndex0 == 1)
            CopyRoutine = Interface1GetHIDDescriptor;
        else
            CopyRoutine = Interface2GetHIDDescriptor;
    }

  } /* End of GET_DESCRIPTOR */

  /*** GET_PROTOCOL ***/
  else if ((Type_Recipient == (CLASS_REQUEST | INTERFACE_RECIPIENT))
           && RequestNo == GET_PROTOCOL)
  {
    CopyRoutine = USBD_GetProtocolValue;
  }


  if (CopyRoutine == NULL)
  {
    return USB_UNSUPPORT;
  }

  pInformation->Ctrl_Info.CopyData = CopyRoutine;
  pInformation->Ctrl_Info.Usb_wOffset = 0;
  (*CopyRoutine)(0);
  return USB_SUCCESS;
}

/*******************************************************************************
* Function Name  : USBD_NoData_Setup.
* Description    : handle the no data class specific requests.
* Input          : Request Nb.
* Return         : USB_UNSUPPORT or USB_SUCCESS.
*******************************************************************************/
RESULT USBD_NoData_Setup(uint8_t RequestNo)
{    
	 if (Type_Recipient == (CLASS_REQUEST | INTERFACE_RECIPIENT))
  {
    if (RequestNo == SET_COMM_FEATURE)
    {
      return USB_SUCCESS;
    }
    else if (RequestNo == SET_CONTROL_LINE_STATE)
    {
      return USB_SUCCESS;
    }
  }

  return USB_SUCCESS;
}

/*******************************************************************************
* Function Name  : Virtual_Com_Port_GetLineCoding.
* Description    : send the linecoding structure to the PC host.
* Input          : Length.
* Return         : Linecoding structure base address.
*******************************************************************************/
uint8_t *Virtual_Com_Port_GetLineCoding(uint16_t Length)
{
  return 0;
}

/*******************************************************************************
* Function Name  : Virtual_Com_Port_SetLineCoding.
* Description    : Set the linecoding structure fields.
* Input          : Length.
* Return         : Linecoding structure base address.
*******************************************************************************/
uint8_t *Virtual_Com_Port_SetLineCoding(uint16_t Length)
{
  return 0;
}
u8 INIT_OK = 0;
/*******************************************************************************
* Function Name  : USBD_GetReportDescriptor.
* Description    : Gets the HID report descriptor.
* Input          : Length
* Output         : None.
* Return         : The address of the configuration descriptor.
*******************************************************************************/
//u8 *KP_GetReportDescriptor(u16 Length)
//{
//  return Standard_GetDescriptorData(Length, &KP_Report_Descriptor);
//}

u8 *Interface0GetReportDescriptor(u16 Length)
{
  return Standard_GetDescriptorData(Length, &Interface0Report_Descriptor);
}
u8 *Interface1GetReportDescriptor(u16 Length)
{
  return Standard_GetDescriptorData(Length, &Interface1Report_Descriptor);
}
u8 *Interface2GetReportDescriptor(u16 Length)
{
  return Standard_GetDescriptorData(Length, &Interface2Report_Descriptor);
}

/*******************************************************************************
* Function Name  : USBD_GetHIDDescriptor.
* Description    : Gets the HID descriptor.
* Input          : Length
* Output         : None.
* Return         : The address of the configuration descriptor.
*******************************************************************************/
u8 *Interface0GetHIDDescriptor(u16 Length)
{
  return Standard_GetDescriptorData(Length, &Interface0Hid_Descriptor);
}
u8 *Interface1GetHIDDescriptor(u16 Length)
{
  return Standard_GetDescriptorData(Length, &Interface1Hid_Descriptor);
}
u8 *Interface2GetHIDDescriptor(u16 Length)
{
  return Standard_GetDescriptorData(Length, &Interface2Hid_Descriptor);
}

/*******************************************************************************
* Function Name  : USBD_SetProtocol
* Description    : USBD Set Protocol request routine.
* Input          : None.
* Output         : None.
* Return         : USB SUCCESS.
*******************************************************************************/
RESULT USBD_SetProtocol(void)
{
  u8 wValue0 = pInformation->USBwValue0;
  ProtocolValue = wValue0;
  return USB_SUCCESS;
}

/*******************************************************************************
* Function Name  : USBD_GetProtocolValue
* Description    : get the protocol value
* Input          : Length.
* Output         : None.
* Return         : address of the protcol value.
*******************************************************************************/
u8 *USBD_GetProtocolValue(u16 Length)
{
  if (Length == 0)
  {
    pInformation->Ctrl_Info.Usb_wLength = 1;
    return NULL;
  }
  else
  {
    return (u8 *)(&ProtocolValue);
  }
}




